@@ -9,7 +9,7 @@
  */
 
 /*
- * $Id: main-hildon.c,v 1.7 2005/12/31 13:14:20 florian Exp $
+ * $Id: main-hildon.c,v 1.9 2006/06/10 15:57:29 florian Exp $
  *
  */
 
@@ -74,11 +74,11 @@ gboolean scroll_delay = FALSE;
 gboolean do_wrap = FALSE;
 static osso_context_t *context = NULL;
 
-#define DEFAULT_STRUCTURE "/var/lib/install/usr/share/gpe-contacts/default-layout.xml"
-#define LARGE_STRUCTURE "/var/lib/install/usr/share/gpe-contacts/default-layout-bigscreen.xml"
+#define DEFAULT_STRUCTURE PREFIX "/share/gpe-contacts/default-layout.xml"
+#define LARGE_STRUCTURE PREFIX "/share/gpe-contacts/default-layout-bigscreen.xml"
 
 struct gpe_icon my_icons[] = {
-  {"icon", "/var/lib/install/usr/share/pixmaps/gpe-contacts.png" },
+  {"icon", PREFIX "/share/pixmaps/gpe-contacts.png" },
   {NULL, NULL}
 };
 
@@ -460,12 +460,12 @@ build_children (GtkWidget *vbox, GSList *children, struct contacts_person *p)
                 if (year)
                   { 
                     tm.tm_year = year - 1900;
-                    tm.tm_mon = month;
+                    tm.tm_mon = month - 1;
                     tm.tm_mday = day;
                     strftime(buf, sizeof(buf), "%x", &tm);
                   }
                 else 
-                  snprintf(buf,sizeof(buf),"%02d/%02d", month+1, day);
+                  snprintf(buf,sizeof(buf),"%02d/%02d", month, day);
                 w = gtk_label_new(buf);
                 gtk_misc_set_alignment(GTK_MISC(w), 0.0, 0.5);
                 hbox = gtk_hbox_new(FALSE, gpe_get_boxspacing());
@@ -840,7 +840,8 @@ do_search (GObject *obj, GtkWidget *entry)
       GtkTreeIter titer;
       
       gtk_list_store_append (list_store, &titer);
-      gtk_list_store_set (list_store, &titer, 0, p->name, 1, p->id, -1);
+      gtk_list_store_set (list_store, &titer, 0, 
+                          (p->name[0] != '(') ? p->name : p->company, 1, p->id, -1);
       contacts_discard_person (p);
     }
   
@@ -895,7 +896,8 @@ do_find (void)
       if (match_for_search (p, NULL, c))
         {
 	      gtk_list_store_append (list_store, &titer);
-	      gtk_list_store_set (list_store, &titer, 0, p->name, 1, p->id, -1);
+          gtk_list_store_set (list_store, &titer, 0, 
+                              (p->name[0] != '(') ? p->name : p->company, 1, p->id, -1);
         }
 
       contacts_discard_person (p);
