@@ -9,7 +9,7 @@
  */
 
 /*
- * $Id: main.c,v 1.124 2005/12/28 23:24:49 florian Exp $
+ * $Id: main.c,v 1.127 2006/06/10 15:57:29 florian Exp $
  *
  */
 
@@ -397,12 +397,12 @@ build_children (GtkWidget *vbox, GSList *children, struct contacts_person *p)
                 if (year)
                   { 
                     tm.tm_year = year - 1900;
-                    tm.tm_mon = month;
+                    tm.tm_mon = month - 1;
                     tm.tm_mday = day;
                     strftime(buf, sizeof(buf), "%x", &tm);
                   }
                 else 
-                  snprintf(buf,sizeof(buf),"%02d/%02d", month+1, day);
+                  snprintf(buf,sizeof(buf),"%02d/%02d", month, day);
                 w = gtk_label_new(buf);
                 gtk_misc_set_alignment(GTK_MISC(w), 0.0, 0.5);
                 hbox = gtk_hbox_new(FALSE, gpe_get_boxspacing());
@@ -785,7 +785,8 @@ do_search (GObject *obj, GtkWidget *entry)
       GtkTreeIter titer;
       
       gtk_list_store_append (list_store, &titer);
-      gtk_list_store_set (list_store, &titer, 0, p->name, 1, p->id, -1);
+      gtk_list_store_set (list_store, &titer, 0, 
+                          (p->name[0] != '(') ? p->name : p->company, 1, p->id, -1);
 
       contacts_discard_person (p);
     }
@@ -843,7 +844,8 @@ do_find (void)
       if (match_for_search (p, NULL, c))
         {
 	      gtk_list_store_append (list_store, &titer);
-	      gtk_list_store_set (list_store, &titer, 0, p->name, 1, p->id, -1);
+	      gtk_list_store_set (list_store, &titer, 0, 
+                              (p->name[0] != '(') ? p->name : p->company, 1, p->id, -1);
         }
 
       contacts_discard_person (p);
@@ -1178,7 +1180,6 @@ on_main_focus(GtkWindow *window,  GdkEventExpose *event,gpointer user_data)
 {
   GtkWidget *searchentry = user_data;
   
-  gtk_widget_grab_focus(searchentry);
   gtk_editable_select_region(GTK_EDITABLE(searchentry),0,-1);
 }
 
@@ -1267,7 +1268,7 @@ create_main (gboolean edit_structure)
   
   if (mode_large_screen)
     {
-       size_x = 620;
+       size_x = 720;
        size_y = 400;
     }
   else
@@ -1651,7 +1652,7 @@ main (int argc, char *argv[])
 
   gpe_set_window_icon (mainw, "icon");
   gtk_widget_show_all (mainw);
-  gtk_widget_grab_focus (GTK_WIDGET (g_object_get_data (G_OBJECT (mainw), "entry")));
+  gtk_widget_grab_focus (GTK_WIDGET (mainw));
 
   path = gtk_tree_path_new_first();
   gtk_tree_view_set_cursor(GTK_TREE_VIEW(list_view), path, NULL, FALSE);
